from __future__ import unicode_literals

from django.contrib import admin

from .models import RecentSearch


@admin.register(RecentSearch)
class RecentSearchAdmin(admin.ModelAdmin):
    date_hierarchy = 'datetime_created'
    list_display = ('user', 'query', 'datetime_created', 'hits')
    list_display_links = ('user', 'query', 'datetime_created', 'hits')
    list_filter = ('user',)
    readonly_fields = ('user', 'query', 'datetime_created', 'hits')



from __future__ import unicode_literals

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.core.exceptions import PermissionDenied
from django.test import TestCase

from user_management.tests import TEST_GROUP, TEST_USER_USERNAME

from ..classes import Permission
from ..models import Role
from ..permissions import permission_role_view

from .literals import TEST_ROLE_LABEL


class PermissionTestCase(TestCase):
    def setUp(self):
        self.user = get_user_model().objects.create(
            username=TEST_USER_USERNAME
        )
        self.group = Group.objects.create(name=TEST_GROUP)
        self.role = Role.objects.create(label=TEST_ROLE_LABEL)
        Permission.invalidate_cache()

    def test_no_permissions(self):
        with self.assertRaises(PermissionDenied):
            Permission.check_permissions(
                requester=self.user, permissions=(permission_role_view,)
            )

    def test_with_permissions(self):
        self.group.user_set.add(self.user)
        self.role.permissions.add(permission_role_view.stored_permission)
        self.role.groups.add(self.group)

        try:
            Permission.check_permissions(
                requester=self.user, permissions=(permission_role_view,)
            )
        except PermissionDenied:
            self.fail('PermissionDenied exception was not expected.')



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from common import MayanAppConfig, menu_tools, menu_object, menu_secondary
from navigation import SourceColumn

from .classes import Property, PropertyNamespace, PIPNotFound, VirtualEnv
from .links import (
    link_menu_link, link_namespace_details, link_namespace_list
)


class InstallationApp(MayanAppConfig):
    name = 'installation'
    verbose_name = _('Installation')

    def ready(self):
        super(InstallationApp, self).ready()

        SourceColumn(
            source=PropertyNamespace, label=_('Label'), attribute='label'
        )
        SourceColumn(
            source=PropertyNamespace, label=_('Items'),
            func=lambda context: len(context['object'].get_properties())
        )

        SourceColumn(source=Property, label=_('Label'), attribute='label')
        SourceColumn(source=Property, label=_('Value'), attribute='value')

        menu_object.bind_links(
            links=(link_namespace_details,), sources=(PropertyNamespace,)
        )
        menu_secondary.bind_links(
            links=(link_namespace_list,),
            sources=('installation:namespace_list', PropertyNamespace)
        )
        menu_tools.bind_links(links=(link_menu_link,))

        namespace = PropertyNamespace('venv', _('VirtualEnv'))
        try:
            venv = VirtualEnv()
        except PIPNotFound:
            namespace.add_property(
                'pip', 'pip', _('pip not found.'), report=True
            )
        else:
            for item, version, result in venv.get_results():
                namespace.add_property(
                    item, '%s (%s)' % (item, version), result, report=True
                )



from __future__ import absolute_import, unicode_literals

from django.core.exceptions import PermissionDenied
from django.shortcuts import get_object_or_404

from rest_framework import generics, status, views
from rest_framework.response import Response

from acls.models import AccessControlList
from documents.models import Document, DocumentType
from documents.permissions import (
    permission_document_type_view, permission_document_type_edit
)
from permissions import Permission
from rest_api.filters import MayanObjectPermissionsFilter
from rest_api.permissions import MayanPermission

from .models import DocumentMetadata, DocumentTypeMetadataType, MetadataType
from .permissions import (
    permission_metadata_document_add, permission_metadata_document_remove,
    permission_metadata_document_edit, permission_metadata_document_view,
    permission_metadata_type_create, permission_metadata_type_delete,
    permission_metadata_type_edit, permission_metadata_type_view
)
from .serializers import (
    DocumentMetadataSerializer, DocumentNewMetadataSerializer,
    DocumentTypeNewMetadataTypeSerializer, MetadataTypeSerializer,
    DocumentTypeMetadataTypeSerializer
)


class APIMetadataTypeListView(generics.ListCreateAPIView):
    serializer_class = MetadataTypeSerializer
    queryset = MetadataType.objects.all()

    permission_classes = (MayanPermission,)
    filter_backends = (MayanObjectPermissionsFilter,)
    mayan_object_permissions = {'GET': (permission_metadata_type_view,)}
    mayan_view_permissions = {'POST': (permission_metadata_type_create,)}

    def get(self, *args, **kwargs):
        """
        Returns a list of all the metadata types.
        """
        return super(APIMetadataTypeListView, self).get(*args, **kwargs)

    def post(self, *args, **kwargs):
        """
        Create a new metadata type.
        """
        return super(APIMetadataTypeListView, self).post(*args, **kwargs)


class APIMetadataTypeView(generics.RetrieveUpdateDestroyAPIView):
    serializer_class = MetadataTypeSerializer
    queryset = MetadataType.objects.all()

    permission_classes = (MayanPermission,)
    mayan_object_permissions = {
        'GET': (permission_metadata_type_view,),
        'PUT': (permission_metadata_type_edit,),
        'PATCH': (permission_metadata_type_edit,),
        'DELETE': (permission_metadata_type_delete,)
    }

    def delete(self, *args, **kwargs):
        """
        Delete the selected metadata type.
        """
        return super(APIMetadataTypeView, self).delete(*args, **kwargs)

    def get(self, *args, **kwargs):
        """
        Return the details of the selected metadata type.
        """
        return super(APIMetadataTypeView, self).get(*args, **kwargs)

    def patch(self, *args, **kwargs):
        """
        Edit the selected metadata type.
        """
        return super(APIMetadataTypeView, self).patch(*args, **kwargs)

    def put(self, *args, **kwargs):
        """
        Edit the selected metadata type.
        """
        return super(APIMetadataTypeView, self).put(*args, **kwargs)


class APIDocumentMetadataListView(generics.ListCreateAPIView):
    permission_classes = (MayanPermission,)

    def get_document(self):
        return get_object_or_404(Document, pk=self.kwargs['pk'])

    def get_queryset(self):
        document = self.get_document()

        if self.request.method == 'GET':
            # Make sure the use has the permission to see the metadata for
            # this document
            try:
                Permission.check_permissions(
                    self.request.user, (permission_metadata_document_view,)
                )
            except PermissionDenied:
                AccessControlList.objects.check_access(
                    permission_metadata_document_view, self.request.user,
                    document
                )
            else:
                return document.metadata.all()
        elif self.request.method == 'POST':
            # Make sure the use has the permission to add metadata to this
            # document
            try:
                Permission.check_permissions(
                    self.request.user, (permission_metadata_document_add,)
                )
            except PermissionDenied:
                AccessControlList.objects.check_access(
                    permission_metadata_document_add, self.request.user,
                    document
                )
            else:
                return document.metadata.all()

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return DocumentMetadataSerializer
        elif self.request.method == 'POST':
            return DocumentNewMetadataSerializer

    def get(self, *args, **kwargs):
        """
        Returns a list of selected document's metadata types and values.
        """
        return super(APIDocumentMetadataListView, self).get(*args, **kwargs)

    def perform_create(self, serializer):
        serializer.document = self.get_document()
        serializer.save()

    def post(self, *args, **kwargs):
        """
        Add an existing metadata type and value to the selected document.
        """
        return super(APIDocumentMetadataListView, self).post(*args, **kwargs)


class APIDocumentMetadataView(generics.RetrieveUpdateDestroyAPIView):
    serializer_class = DocumentMetadataSerializer
    queryset = DocumentMetadata.objects.all()

    permission_classes = (MayanPermission,)
    mayan_object_permissions = {
        'GET': (permission_metadata_document_view,),
        'PUT': (permission_metadata_document_edit,),
        'PATCH': (permission_metadata_document_edit,),
        'DELETE': (permission_metadata_document_remove,)
    }

    def delete(self, *args, **kwargs):
        """
        Delete the selected document metadata type and value.
        """
        try:
            return super(
                APIDocumentMetadataView, self
            ).delete(*args, **kwargs)
        except Exception as exception:
            return Response(
                status=status.HTTP_400_BAD_REQUEST, data={
                    'non_fields_errors': unicode(exception)
                }
            )

    def get(self, *args, **kwargs):
        """
        Return the details of the selected document metadata type and value.
        """
        return super(APIDocumentMetadataView, self).get(*args, **kwargs)

    def patch(self, *args, **kwargs):
        """
        Edit the selected document metadata type and value.
        """
        try:
            return super(
                APIDocumentMetadataView, self
            ).patch(*args, **kwargs)
        except Exception as exception:
            return Response(
                status=status.HTTP_400_BAD_REQUEST, data={
                    'non_fields_errors': unicode(exception)
                }
            )

    def put(self, *args, **kwargs):
        """
        Edit the selected document metadata type and value.
        """
        try:
            return super(APIDocumentMetadataView, self).put(*args, **kwargs)
        except Exception as exception:
            return Response(
                status=status.HTTP_400_BAD_REQUEST, data={
                    'non_fields_errors': unicode(exception)
                }
            )


class APIDocumentTypeMetadataTypeOptionalListView(generics.ListCreateAPIView):
    permission_classes = (MayanPermission,)

    mayan_view_permissions = {'POST': (permission_document_type_edit,)}

    required_metadata = False

    def get_queryset(self):
        document_type = get_object_or_404(
            DocumentType, pk=self.kwargs['document_type_pk']
        )
        try:
            Permission.check_permissions(
                self.request.user, (permission_document_type_view,)
            )
        except PermissionDenied:
            AccessControlList.objects.check_access(
                permission_document_type_view, self.request.user,
                document_type
            )

        return document_type.metadata.filter(required=self.required_metadata)

    def get(self, *args, **kwargs):
        """
        Returns a list of selected document type's optional metadata types.
        """
        return super(
            APIDocumentTypeMetadataTypeOptionalListView, self
        ).get(*args, **kwargs)

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return DocumentTypeMetadataTypeSerializer
        elif self.request.method == 'POST':
            return DocumentTypeNewMetadataTypeSerializer

    def post(self, request, *args, **kwargs):
        """
        Add an optional metadata type to a document type.
        """
        document_type = get_object_or_404(
            DocumentType, pk=self.kwargs['document_type_pk']
        )

        try:
            Permission.check_permissions(
                self.request.user, (permission_document_type_edit,)
            )
        except PermissionDenied:
            AccessControlList.objects.check_access(
                permission_document_type_edit, self.request.user,
                document_type
            )

        serializer = self.get_serializer(data=self.request.POST)

        if serializer.is_valid():
            metadata_type = get_object_or_404(
                MetadataType, pk=serializer.data['metadata_type_pk']
            )
            document_type_metadata_type = document_type.metadata.create(
                metadata_type=metadata_type, required=self.required_metadata
            )
            return Response(
                status=status.HTTP_201_CREATED,
                data={
                    'pk': document_type_metadata_type.pk
                }
            )
        else:
            return Response(status=status.HTTP_400_BAD_REQUEST)


class APIDocumentTypeMetadataTypeRequiredListView(APIDocumentTypeMetadataTypeOptionalListView):
    required_metadata = True

    def get(self, *args, **kwargs):
        """
        Returns a list of the selected document type's required metadata
        types.
        """
        return super(
            APIDocumentTypeMetadataTypeRequiredListView, self
        ).get(*args, **kwargs)

    def post(self, request, *args, **kwargs):
        """
        Add a required metadata type to a document type.
        """
        return super(
            APIDocumentTypeMetadataTypeRequiredListView, self
        ).post(request, *args, **kwargs)


class APIDocumentTypeMetadataTypeView(views.APIView):
    def delete(self, request, *args, **kwargs):
        """
        Remove a metadata type from a document type.
        """

        document_type_metadata_type = get_object_or_404(
            DocumentTypeMetadataType, pk=self.kwargs['pk']
        )

        try:
            Permission.check_permissions(
                self.request.user, (permission_document_type_edit,)
            )
        except PermissionDenied:
            AccessControlList.objects.check_access(
                permission_document_type_edit, self.request.user,
                document_type_metadata_type.document_type
            )

        document_type_metadata_type.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



from __future__ import unicode_literals

from django.conf.urls import patterns, url

from .views import (
    DocumentWorkflowInstanceListView, SetupWorkflowCreateView,
    SetupWorkflowDeleteView, SetupWorkflowDocumentTypesView,
    SetupWorkflowEditView, SetupWorkflowListView,
    SetupWorkflowStateCreateView, SetupWorkflowStateDeleteView,
    SetupWorkflowStateEditView, SetupWorkflowStateListView,
    SetupWorkflowTransitionListView, SetupWorkflowTransitionCreateView,
    SetupWorkflowTransitionDeleteView, SetupWorkflowTransitionEditView,
    WorkflowDocumentListView, WorkflowInstanceDetailView,
    WorkflowInstanceTransitionView
)

urlpatterns = patterns(
    '',
    url(
        r'^document/(?P<pk>\d+)/workflows/$',
        DocumentWorkflowInstanceListView.as_view(),
        name='document_workflow_instance_list'
    ),
    url(
        r'^document/workflows/(?P<pk>\d+)/$',
        WorkflowInstanceDetailView.as_view(), name='workflow_instance_detail'
    ),
    url(
        r'^document/workflows/(?P<pk>\d+)/transition/$',
        WorkflowInstanceTransitionView.as_view(),
        name='workflow_instance_transition'
    ),

    url(
        r'^setup/all/$', SetupWorkflowListView.as_view(),
        name='setup_workflow_list'
    ),
    url(
        r'^setup/create/$', SetupWorkflowCreateView.as_view(),
        name='setup_workflow_create'
    ),
    url(
        r'^setup/(?P<pk>\d+)/edit/$', SetupWorkflowEditView.as_view(),
        name='setup_workflow_edit'
    ),
    url(
        r'^setup/(?P<pk>\d+)/delete/$', SetupWorkflowDeleteView.as_view(),
        name='setup_workflow_delete'
    ),
    url(
        r'^setup/(?P<pk>\d+)/documents/$',
        WorkflowDocumentListView.as_view(),
        name='setup_workflow_document_list'
    ),
    url(
        r'^setup/(?P<pk>\d+)/document_types/$',
        SetupWorkflowDocumentTypesView.as_view(),
        name='setup_workflow_document_types'
    ),
    url(
        r'^setup/(?P<pk>\d+)/states/$', SetupWorkflowStateListView.as_view(),
        name='setup_workflow_states'
    ),
    url(
        r'^setup/(?P<pk>\d+)/states/create/$',
        SetupWorkflowStateCreateView.as_view(),
        name='setup_workflow_state_create'
    ),
    url(
        r'^setup/(?P<pk>\d+)/transitions/$',
        SetupWorkflowTransitionListView.as_view(),
        name='setup_workflow_transitions'
    ),
    url(
        r'^setup/(?P<pk>\d+)/transitions/create/$',
        SetupWorkflowTransitionCreateView.as_view(),
        name='setup_workflow_transition_create'
    ),
    url(
        r'^setup/workflow/state/(?P<pk>\d+)/delete/$',
        SetupWorkflowStateDeleteView.as_view(),
        name='setup_workflow_state_delete'
    ),
    url(
        r'^setup/workflow/state/(?P<pk>\d+)/edit/$',
        SetupWorkflowStateEditView.as_view(),
        name='setup_workflow_state_edit'
    ),
    url(
        r'^setup/workflow/transitions/(?P<pk>\d+)/delete/$',
        SetupWorkflowTransitionDeleteView.as_view(),
        name='setup_workflow_transition_delete'
    ),
    url(
        r'^setup/workflow/transitions/(?P<pk>\d+)/edit/$',
        SetupWorkflowTransitionEditView.as_view(),
        name='setup_workflow_transition_edit'
    ),
)



from __future__ import unicode_literals

from django.db.models import get_model


def launch_workflow(sender, instance, created, **kwargs):
    Workflow = get_model('document_states', 'Workflow')

    if created:
        Workflow.objects.launch_for(instance)



from __future__ import unicode_literals


class DocumentException(Exception):
    """
    Base documents warning
    """
    pass


class NewDocumentVersionNotAllowed(DocumentException):
    """
    Uploading new versions for this document is not allowed
    """
    pass



from __future__ import absolute_import, unicode_literals

from django.core.exceptions import PermissionDenied

from rest_framework.filters import BaseFilterBackend

from acls.models import AccessControlList
from permissions import Permission


class MayanObjectPermissionsFilter(BaseFilterBackend):
    def filter_queryset(self, request, queryset, view):
        required_permission = getattr(
            view, 'mayan_object_permissions', {}
        ).get(request.method, None)

        if required_permission:
            try:
                Permission.check_permissions(request.user, required_permission)
            except PermissionDenied:
                return AccessControlList.objects.filter_by_access(
                    required_permission[0], request.user, queryset
                )
            else:
                return queryset
        else:
            return queryset



from __future__ import unicode_literals

from django.test import TestCase

from .mixins import ContentTypeCheckMixin, OpenFileCheckMixin, TempfileCheckMixin


class BaseTestCase(ContentTypeCheckMixin, OpenFileCheckMixin, TempfileCheckMixin, TestCase):
    """
    This is the most basic test case class any test in the project should use.
    """



from __future__ import absolute_import

from __future__ import unicode_literals

from django.core.exceptions import PermissionDenied

from rest_framework.permissions import BasePermission

from acls.models import AccessControlList
from permissions import Permission


class MayanPermission(BasePermission):
    def has_permission(self, request, view):
        required_permission = getattr(
            view, 'mayan_view_permissions', {}
        ).get(request.method, None)

        if required_permission:
            try:
                Permission.check_permissions(request.user, required_permission)
            except PermissionDenied:
                return False
            else:
                return True
        else:
            return True

    def has_object_permission(self, request, view, obj):
        required_permission = getattr(
            view, 'mayan_object_permissions', {}
        ).get(request.method, None)

        if required_permission:
            try:
                Permission.check_permissions(request.user, required_permission)
            except PermissionDenied:
                try:
                    if hasattr(view, 'mayan_permission_attribute_check'):
                        AccessControlList.objects.check_access(
                            permissions=required_permission,
                            user=request.user, obj=obj,
                            related=view.mayan_permission_attribute_check
                        )
                    else:
                        AccessControlList.objects.check_access(
                            required_permission, request.user, obj
                        )
                except PermissionDenied:
                    return False
                else:
                    return True
            else:
                return True
        else:
            return True



from __future__ import absolute_import, unicode_literals

from ..base import *  # NOQA

SIGNATURES_GPG_PATH = '/usr/bin/gpg1'

INSTALLED_APPS += ('test_without_migrations',)
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader'
)



from __future__ import unicode_literals

from rest_framework.filters import BaseFilterBackend


class RecentSearchUserFilter(BaseFilterBackend):
    def filter_queryset(self, request, queryset, view):
        if request.user.is_staff or request.user.is_superuser:
            return queryset
        else:
            return queryset.filter(user=self.request.user)



from __future__ import unicode_literals

from django.conf.urls import patterns, url

from .views import (
    MessageCreateView, MessageDeleteView, MessageEditView, MessageListView
)

urlpatterns = patterns(
    '',
    url(r'^list/$', MessageListView.as_view(), name='message_list'),
    url(r'^create/$', MessageCreateView.as_view(), name='message_create'),
    url(r'^(?P<pk>\d+)/edit/$', MessageEditView.as_view(), name='message_edit'),
    url(
        r'^(?P<pk>\d+)/delete/$', MessageDeleteView.as_view(),
        name='message_delete'
    ),
)



from __future__ import absolute_import, unicode_literals

from django.core.exceptions import PermissionDenied
from django.shortcuts import get_object_or_404

from rest_framework import generics
from rest_framework.response import Response

from acls.models import AccessControlList
from documents.models import Document
from documents.permissions import permission_document_view
from permissions import Permission
from rest_api.filters import MayanObjectPermissionsFilter
from rest_api.permissions import MayanPermission

from .models import Folder
from .permissions import (
    permission_folder_add_document, permission_folder_create,
    permission_folder_delete, permission_folder_edit,
    permission_folder_remove_document, permission_folder_view
)
from .serializers import (
    FolderDocumentSerializer, FolderSerializer, NewFolderDocumentSerializer,
    NewFolderSerializer
)


class APIDocumentFolderListView(generics.ListAPIView):
    """
    Returns a list of all the folders to which a document belongs.
    """

    serializer_class = FolderSerializer

    filter_backends = (MayanObjectPermissionsFilter,)
    mayan_object_permissions = {'GET': (permission_folder_view,)}

    def get_queryset(self):
        document = get_object_or_404(Document, pk=self.kwargs['pk'])
        try:
            Permission.check_permissions(
                self.request.user, (permission_document_view,)
            )
        except PermissionDenied:
            AccessControlList.objects.check_access(
                permission_document_view, self.request.user, document
            )

        queryset = document.document_folders().all()
        return queryset


class APIFolderListView(generics.ListCreateAPIView):
    filter_backends = (MayanObjectPermissionsFilter,)
    mayan_object_permissions = {'GET': (permission_folder_view,)}
    mayan_view_permissions = {'POST': (permission_folder_create,)}
    permission_classes = (MayanPermission,)
    queryset = Folder.objects.all()

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return FolderSerializer
        elif self.request.method == 'POST':
            return NewFolderSerializer

    def get(self, *args, **kwargs):
        """
        Returns a list of all the folders.
        """
        return super(APIFolderListView, self).get(*args, **kwargs)

    def post(self, *args, **kwargs):
        """
        Create a new folder.
        """
        return super(APIFolderListView, self).post(*args, **kwargs)


class APIFolderView(generics.RetrieveUpdateDestroyAPIView):
    filter_backends = (MayanObjectPermissionsFilter,)
    mayan_object_permissions = {
        'GET': (permission_folder_view,),
        'PUT': (permission_folder_edit,),
        'PATCH': (permission_folder_edit,),
        'DELETE': (permission_folder_delete,)
    }
    permission_classes = (MayanPermission,)
    queryset = Folder.objects.all()
    serializer_class = FolderSerializer

    def delete(self, *args, **kwargs):
        """
        Delete the selected folder.
        """
        return super(APIFolderView, self).delete(*args, **kwargs)

    def get(self, *args, **kwargs):
        """
        Returns the details of the selected folder.
        """
        return super(APIFolderView, self).get(*args, **kwargs)

    def patch(self, *args, **kwargs):
        """
        Edit the selected folder.
        """
        return super(APIFolderView, self).patch(*args, **kwargs)

    def put(self, *args, **kwargs):
        """
        Edit the selected folder.
        """
        return super(APIFolderView, self).put(*args, **kwargs)


class APIFolderDocumentListView(generics.ListCreateAPIView):
    """
    Returns a list of all the documents contained in a particular folder.
    """

    filter_backends = (MayanObjectPermissionsFilter,)
    mayan_object_permissions = {
        'GET': (permission_folder_view,),
        'POST': (permission_folder_add_document,)
    }

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return FolderDocumentSerializer
        elif self.request.method == 'POST':
            return NewFolderDocumentSerializer

    def get_serializer_context(self):
        """
        Extra context provided to the serializer class.
        """
        return {
            'folder': self.get_folder(),
            'format': self.format_kwarg,
            'request': self.request,
            'view': self
        }

    def get_folder(self):
        return get_object_or_404(Folder, pk=self.kwargs['pk'])

    def get_queryset(self):
        folder = self.get_folder()

        documents = folder.documents.all()

        try:
            Permission.check_permissions(
                self.request.user, (permission_document_view,)
            )
        except PermissionDenied:
            documents = AccessControlList.objects.filter_by_access(
                permission_document_view, self.request.user, documents
            )

        return documents

    def perform_create(self, serializer):
        serializer.save(folder=self.get_folder())

    def post(self, request, *args, **kwargs):
        """
        Add a document to the selected folder.
        """
        return super(APIFolderDocumentListView, self).post(request, *args, **kwargs)


class APIFolderDocumentView(generics.RetrieveDestroyAPIView):
    filter_backends = (MayanObjectPermissionsFilter,)
    mayan_object_permissions = {
        'GET': (permission_folder_view,),
        'DELETE': (permission_folder_remove_document,)
    }
    serializer_class = FolderDocumentSerializer

    def delete(self, request, *args, **kwargs):
        """
        Remove a document from the selected folder.
        """

        return super(APIFolderDocumentView, self).delete(request, *args, **kwargs)

    def get(self, *args, **kwargs):
        """
        Returns the details of the selected folder document.
        """

        return super(APIFolderDocumentView, self).get(*args, **kwargs)

    def get_folder(self):
        return get_object_or_404(Folder, pk=self.kwargs['folder_pk'])

    def get_queryset(self):
        return self.get_folder().documents.all()

    def get_serializer_context(self):
        """
        Extra context provided to the serializer class.
        """
        return {
            'folder': self.get_folder(),
            'format': self.format_kwarg,
            'request': self.request,
            'view': self
        }

    def perform_destroy(self, instance):
        self.get_folder().documents.remove(instance)

    def retrieve(self, request, *args, **kwargs):
        instance = self.get_object()

        try:
            Permission.check_permissions(
                self.request.user, (permission_document_view,)
            )
        except PermissionDenied:
            AccessControlList.objects.check_access(
                permission_document_view, self.request.user, instance
            )

        serializer = self.get_serializer(instance)
        return Response(serializer.data)



from __future__ import unicode_literals

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group

from rest_framework import serializers


class GroupSerializer(serializers.HyperlinkedModelSerializer):
    users_count = serializers.SerializerMethodField()

    class Meta:
        extra_kwargs = {
            'url': {'view_name': 'rest_api:group-detail'}
        }
        fields = ('id', 'name', 'url', 'users_count')
        model = Group

    def get_users_count(self, instance):
        return instance.user_set.count()


class UserSerializer(serializers.HyperlinkedModelSerializer):
    groups = GroupSerializer(many=True)

    password = serializers.CharField(
        required=False, style={'input_type': 'password'}
    )

    class Meta:
        extra_kwargs = {
            'url': {'view_name': 'rest_api:user-detail'}
        }
        fields = (
            'first_name', 'date_joined', 'email', 'groups', 'id', 'is_active',
            'last_login', 'last_name', 'url', 'username', 'password'
        )
        model = get_user_model()
        read_only_fields = ('last_login', 'date_joined')
        write_only_fields = ('password',)

    def create(self, validated_data):
        validated_data.pop('groups')
        validated_data.pop('is_active')
        user = get_user_model().objects.create_user(**validated_data)

        return user

    def update(self, instance, validated_data):
        validated_data.pop('groups')

        if 'password' in validated_data:
            instance.set_password(validated_data['password'])
            validated_data.pop('password')

        for attr, value in validated_data.items():
            setattr(instance, attr, value)

        instance.save()

        return instance



from __future__ import unicode_literals

from django.conf.urls import patterns, url

from .views import NamespaceDetailView, NamespaceListView

urlpatterns = patterns(
    'installation.views',
    url(r'^$', NamespaceListView.as_view(), name='namespace_list'),
    url(
        r'^(?P<namespace_id>\w+)/details/$', NamespaceDetailView.as_view(),
        name='namespace_details'
    ),
)



from __future__ import unicode_literals

from django import forms
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _


class UserForm(forms.ModelForm):
    class Meta:
        model = get_user_model()
        fields = ('username', 'first_name', 'last_name', 'email', 'is_active',)


class PasswordForm(forms.Form):
    new_password_1 = forms.CharField(
        label=_('New password'), widget=forms.PasswordInput()
    )
    new_password_2 = forms.CharField(
        label=_('Confirm password'), widget=forms.PasswordInput()
    )



from __future__ import absolute_import, unicode_literals

import logging

from django.conf import settings
from django.contrib import messages
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse, reverse_lazy
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _, ungettext

from acls.models import AccessControlList
from common.views import (
    SingleObjectCreateView, SingleObjectDeleteView, SingleObjectEditView,
    SingleObjectListView
)
from documents.permissions import permission_document_view
from documents.models import Document
from documents.views import DocumentListView
from permissions import Permission

from .forms import FolderListForm
from .models import Folder
from .permissions import (
    permission_folder_add_document, permission_folder_create,
    permission_folder_delete, permission_folder_edit, permission_folder_view,
    permission_folder_remove_document
)

logger = logging.getLogger(__name__)


class FolderCreateView(SingleObjectCreateView):
    fields = ('label',)
    model = Folder
    view_permission = permission_folder_create

    def get_extra_context(self):
        return {
            'title': _('Create folder'),
        }


class FolderDeleteView(SingleObjectDeleteView):
    model = Folder
    object_permission = permission_folder_delete
    post_action_redirect = reverse_lazy('folders:folder_list')

    def get_extra_context(self):
        return {
            'object': self.get_object(),
            'title': _('Delete the folder: %s?') % self.get_object(),
        }


class FolderDetailView(DocumentListView):
    def get_document_queryset(self):
        return self.get_folder().documents.all()

    def get_extra_context(self):
        return {
            'hide_links': True,
            'object': self.get_folder(),
            'title': _('Documents in folder: %s') % self.get_folder(),
        }

    def get_folder(self):
        folder = get_object_or_404(Folder, pk=self.kwargs['pk'])

        try:
            Permission.check_permissions(
                self.request.user, (permission_folder_view,)
            )
        except PermissionDenied:
            AccessControlList.objects.check_access(
                permission_folder_view, self.request.user, folder
            )

        return folder


class FolderEditView(SingleObjectEditView):
    fields = ('label',)
    model = Folder
    object_permission = permission_folder_edit
    post_action_redirect = reverse_lazy('folders:folder_list')

    def get_extra_context(self):
        return {
            'object': self.get_object(),
            'title': _('Edit folder: %s') % self.get_object(),
        }


class FolderListView(SingleObjectListView):
    model = Folder
    object_permission = permission_folder_view

    def get_extra_context(self):
        return {
            'hide_link': True,
            'title': _('Folders'),
        }


class DocumentFolderListView(FolderListView):
    def dispatch(self, request, *args, **kwargs):
        self.document = get_object_or_404(Document, pk=self.kwargs['pk'])

        try:
            Permission.check_permissions(
                request.user, (permission_document_view,)
            )
        except PermissionDenied:
            AccessControlList.objects.check_access(
                permission_document_view, request.user, self.document
            )

        return super(DocumentFolderListView, self).dispatch(request, *args, **kwargs)

    def get_extra_context(self):
        return {
            'hide_link': True,
            'object': self.document,
            'title': _('Folders containing document: %s') % self.document,
        }

    def get_queryset(self):
        return self.document.document_folders().all()


def folder_add_document(request, document_id=None, document_id_list=None):
    if document_id:
        queryset = Document.objects.filter(pk=document_id)
    elif document_id_list:
        queryset = Document.objects.filter(pk__in=document_id_list)

    if not queryset:
        messages.error(request, _('Must provide at least one document.'))
        return HttpResponseRedirect(
            request.META.get(
                'HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL)
            )
        )

    try:
        Permission.check_permissions(
            request.user, (permission_folder_add_document,)
        )
    except PermissionDenied:
        queryset = AccessControlList.objects.filter_by_access(
            permission_folder_add_document, request.user, queryset
        )

    post_action_redirect = None
    if document_id:
        post_action_redirect = reverse(
            'folders:document_folder_list', args=(document_id,)
        )

    previous = request.POST.get('previous', request.GET.get('previous', request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))))
    next = request.POST.get('next', request.GET.get('next', post_action_redirect if post_action_redirect else request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))))

    if request.method == 'POST':
        form = FolderListForm(request.POST, user=request.user)
        if form.is_valid():
            folder = form.cleaned_data['folder']
            for document in queryset:
                if document.pk not in folder.documents.values_list('pk', flat=True):
                    folder.documents.add(document)
                    messages.success(
                        request, _(
                            'Document: %(document)s added to folder: '
                            '%(folder)s successfully.'
                        ) % {
                            'document': document, 'folder': folder
                        }
                    )
                else:
                    messages.warning(
                        request, _(
                            'Document: %(document)s is already in '
                            'folder: %(folder)s.'
                        ) % {
                            'document': document, 'folder': folder
                        }
                    )

            return HttpResponseRedirect(next)
    else:
        form = FolderListForm(user=request.user)

    context = {
        'form': form,
        'previous': previous,
        'next': next,
    }

    if queryset.count() == 1:
        context['object'] = queryset.first()

    context['title'] = ungettext(
        'Add document to folder',
        'Add documents to folder',
        queryset.count()
    )

    return render_to_response(
        'appearance/generic_form.html', context,
        context_instance=RequestContext(request)
    )


def folder_document_remove(request, folder_id, document_id=None, document_id_list=None):
    post_action_redirect = None

    folder = get_object_or_404(Folder, pk=folder_id)

    if document_id:
        queryset = Document.objects.filter(pk=document_id)
    elif document_id_list:
        queryset = Document.objects.filter(pk__in=document_id_list)

    if not queryset:
        messages.error(request, _('Must provide at least one folder document.'))
        return HttpResponseRedirect(request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL)))

    try:
        Permission.check_permissions(
            request.user, (permission_folder_remove_document,)
        )
    except PermissionDenied:
        queryset = AccessControlList.objects.filter_by_access(
            permission_folder_remove_document, request.user, queryset
        )

    previous = request.POST.get('previous', request.GET.get('previous', request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))))
    next = request.POST.get('next', request.GET.get('next', post_action_redirect if post_action_redirect else request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))))

    if request.method == 'POST':
        for folder_document in queryset:
            try:
                folder.documents.remove(folder_document)
                messages.success(
                    request, _(
                        'Document: %s removed successfully.'
                    ) % folder_document
                )
            except Exception as exception:
                messages.error(
                    request, _(
                        'Document: %(document)s delete error: %(error)s'
                    ) % {
                        'document': folder_document, 'error': exception
                    }
                )

        return HttpResponseRedirect(next)

    context = {
        'next': next,
        'object': folder,
        'previous': previous,
        'title': ungettext(
            'Remove the selected document from the folder: %(folder)s?',
            'Remove the selected documents from the folder: %(folder)s?',
            queryset.count()
        ) % {'folder': folder}
    }

    if queryset.count() == 1:
        context['object'] = queryset.first()

    return render_to_response(
        'appearance/generic_confirm.html', context,
        context_instance=RequestContext(request)
    )


def folder_document_multiple_remove(request, folder_id):
    return folder_document_remove(
        request, folder_id, document_id_list=request.GET.get(
            'id_list', request.POST.get('id_list', '')
        ).split(',')
    )


def folder_add_multiple_documents(request):
    return folder_add_document(
        request, document_id_list=request.GET.get(
            'id_list', request.POST.get('id_list', '')
        ).split(',')
    )



from __future__ import unicode_literals

from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse
from django.test import override_settings

from rest_framework.test import APITestCase

from documents.models import DocumentType
from documents.tests import TEST_DOCUMENT_TYPE, TEST_SMALL_DOCUMENT_PATH
from user_management.tests.literals import (
    TEST_ADMIN_EMAIL, TEST_ADMIN_PASSWORD, TEST_ADMIN_USERNAME
)

from ..models import DocumentMetadata, DocumentTypeMetadataType, MetadataType

from .literals import (
    TEST_METADATA_TYPE_LABEL, TEST_METADATA_TYPE_LABEL_2,
    TEST_METADATA_TYPE_NAME, TEST_METADATA_TYPE_NAME_2
)

TEST_METADATA_VALUE = 'test value'
TEST_METADATA_VALUE_EDITED = 'test value edited'


class MetadataTypeAPITestCase(APITestCase):
    def setUp(self):
        self.admin_user = get_user_model().objects.create_superuser(
            username=TEST_ADMIN_USERNAME, email=TEST_ADMIN_EMAIL,
            password=TEST_ADMIN_PASSWORD
        )

        self.client.login(
            username=TEST_ADMIN_USERNAME, password=TEST_ADMIN_PASSWORD
        )

    def tearDown(self):
        self.admin_user.delete()

    def test_metadata_type_create(self):
        response = self.client.post(
            reverse('rest_api:metadatatype-list'), data={
                'label': TEST_METADATA_TYPE_LABEL,
                'name': TEST_METADATA_TYPE_NAME
            }
        )

        metadata_type = MetadataType.objects.first()

        self.assertEqual(response.status_code, 201)
        self.assertEqual(response.data['id'], metadata_type.pk)
        self.assertEqual(response.data['label'], TEST_METADATA_TYPE_LABEL)
        self.assertEqual(response.data['name'], TEST_METADATA_TYPE_NAME)

        self.assertEqual(metadata_type.label, TEST_METADATA_TYPE_LABEL)
        self.assertEqual(metadata_type.name, TEST_METADATA_TYPE_NAME)

    def test_metadata_type_delete(self):
        metadata_type = MetadataType.objects.create(
            label=TEST_METADATA_TYPE_LABEL, name=TEST_METADATA_TYPE_NAME
        )

        response = self.client.delete(
            reverse('rest_api:metadatatype-detail', args=(metadata_type.pk,))
        )

        self.assertEqual(response.status_code, 204)

        self.assertEqual(MetadataType.objects.count(), 0)

    def test_metadata_type_edit(self):
        metadata_type = MetadataType.objects.create(
            label=TEST_METADATA_TYPE_LABEL, name=TEST_METADATA_TYPE_NAME
        )

        response = self.client.put(
            reverse('rest_api:metadatatype-detail', args=(metadata_type.pk,)),
            data={
                'label': TEST_METADATA_TYPE_LABEL_2,
                'name': TEST_METADATA_TYPE_NAME_2
            }
        )

        self.assertEqual(response.status_code, 200)

        metadata_type.refresh_from_db()

        self.assertEqual(metadata_type.label, TEST_METADATA_TYPE_LABEL_2)
        self.assertEqual(metadata_type.name, TEST_METADATA_TYPE_NAME_2)


class DocumentTypeMetadataTypeAPITestCase(APITestCase):
    def setUp(self):
        self.admin_user = get_user_model().objects.create_superuser(
            username=TEST_ADMIN_USERNAME, email=TEST_ADMIN_EMAIL,
            password=TEST_ADMIN_PASSWORD
        )

        self.client.login(
            username=TEST_ADMIN_USERNAME, password=TEST_ADMIN_PASSWORD
        )

        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE
        )

        self.metadata_type = MetadataType.objects.create(
            label=TEST_METADATA_TYPE_LABEL, name=TEST_METADATA_TYPE_NAME
        )

    def tearDown(self):
        self.admin_user.delete()
        self.document_type.delete()

    def test_document_type_metadata_type_optional_create(self):
        response = self.client.post(
            reverse(
                'rest_api:documenttypeoptionalmetadatatype-list',
                args=(self.document_type.pk,)
            ), data={'metadata_type_pk': self.metadata_type.pk}
        )

        self.assertEqual(response.status_code, 201)

        document_type_metadata_type = DocumentTypeMetadataType.objects.filter(
            document_type=self.document_type, required=False
        ).first()

        self.assertEqual(response.data['pk'], document_type_metadata_type.pk)

        self.assertEqual(
            document_type_metadata_type.metadata_type, self.metadata_type
        )

    def test_document_type_metadata_type_required_create(self):
        response = self.client.post(
            reverse(
                'rest_api:documenttyperequiredmetadatatype-list',
                args=(self.document_type.pk,)
            ), data={'metadata_type_pk': self.metadata_type.pk}
        )

        self.assertEqual(response.status_code, 201)

        document_type_metadata_type = DocumentTypeMetadataType.objects.filter(
            document_type=self.document_type, required=True
        ).first()

        self.assertEqual(response.data['pk'], document_type_metadata_type.pk)

        self.assertEqual(
            document_type_metadata_type.metadata_type, self.metadata_type
        )

    def test_document_type_metadata_type_delete(self):
        document_type_metadata_type = self.document_type.metadata.create(
            metadata_type=self.metadata_type, required=True
        )

        response = self.client.delete(
            reverse(
                'rest_api:documenttypemetadatatype-detail',
                args=(document_type_metadata_type.pk,)
            ),
        )

        self.assertEqual(response.status_code, 204)

        self.assertEqual(self.document_type.metadata.all().count(), 0)


class DocumentMetadataAPITestCase(APITestCase):
    @override_settings(OCR_AUTO_OCR=False)
    def setUp(self):
        self.admin_user = get_user_model().objects.create_superuser(
            username=TEST_ADMIN_USERNAME, email=TEST_ADMIN_EMAIL,
            password=TEST_ADMIN_PASSWORD
        )

        self.client.login(
            username=TEST_ADMIN_USERNAME, password=TEST_ADMIN_PASSWORD
        )

        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE
        )

        self.metadata_type = MetadataType.objects.create(
            label=TEST_METADATA_TYPE_LABEL, name=TEST_METADATA_TYPE_NAME
        )

        self.document_type.metadata.create(
            metadata_type=self.metadata_type, required=False
        )

        with open(TEST_SMALL_DOCUMENT_PATH) as file_object:
            self.document = self.document_type.new_document(
                file_object=file_object,
            )

    def tearDown(self):
        self.admin_user.delete()
        self.document_type.delete()

    def test_document_metadata_create(self):
        response = self.client.post(
            reverse(
                'rest_api:documentmetadata-list',
                args=(self.document.pk,)
            ), data={
                'metadata_type_pk': self.metadata_type.pk,
                'value': TEST_METADATA_VALUE
            }
        )

        document_metadata = DocumentMetadata.objects.get(
            document=self.document
        )

        self.assertEqual(response.status_code, 201)

        self.assertEqual(response.data['pk'], document_metadata.pk)

        self.assertEqual(document_metadata.metadata_type, self.metadata_type)
        self.assertEqual(document_metadata.value, TEST_METADATA_VALUE)

    def test_document_metadata_list(self):
        document_metadata = self.document.metadata.create(
            metadata_type=self.metadata_type, value=TEST_METADATA_VALUE
        )

        response = self.client.get(
            reverse(
                'rest_api:documentmetadata-list', args=(self.document.pk,)
            )
        )

        self.assertEqual(response.status_code, 200)

        self.assertEqual(
            response.data['results'][0]['document'], self.document.pk
        )
        self.assertEqual(
            response.data['results'][0]['metadata_type'], self.metadata_type.pk
        )
        self.assertEqual(
            response.data['results'][0]['value'], TEST_METADATA_VALUE
        )
        self.assertEqual(
            response.data['results'][0]['id'], document_metadata.pk
        )

    def test_document_metadata_edit(self):
        document_metadata = self.document.metadata.create(
            metadata_type=self.metadata_type, value=TEST_METADATA_VALUE
        )

        response = self.client.put(
            reverse(
                'rest_api:documentmetadata-detail',
                args=(document_metadata.pk,)
            ), data={
                'value': TEST_METADATA_VALUE_EDITED
            }
        )

        self.assertEqual(response.status_code, 200)

        self.assertEqual(
            response.data['document'], self.document.pk
        )
        self.assertEqual(
            response.data['metadata_type'], self.metadata_type.pk
        )
        self.assertEqual(
            response.data['value'], TEST_METADATA_VALUE_EDITED
        )

    def test_document_metadata_delete(self):
        document_metadata = self.document.metadata.create(
            metadata_type=self.metadata_type, value=TEST_METADATA_VALUE
        )

        response = self.client.delete(
            reverse(
                'rest_api:documentmetadata-detail',
                args=(document_metadata.pk,)
            )
        )

        self.assertEqual(response.status_code, 204)

        self.assertEqual(self.document.metadata.all().count(), 0)



from __future__ import unicode_literals

import logging

import sh

from PIL import Image
import pytesseract

from ..classes import OCRBackendBase
from ..exceptions import OCRError
from ..settings import setting_tesseract_path

logger = logging.getLogger(__name__)


class Tesseract(OCRBackendBase):
    def __init__(self, *args, **kwargs):
        super(Tesseract, self).__init__(*args, **kwargs)
        try:
            self.binary = sh.Command(setting_tesseract_path.value)
        except sh.CommandNotFound:
            self.binary = None

    def get_languages(self):
        if self.binary:
            result = self.binary(list_langs=True)

            return [
                language for language in result.stderr.split('\n') if language
            ]
        else:
            return ()

    def execute(self, *args, **kwargs):
        """
        Execute the command line binary of tesseract
        """
        super(Tesseract, self).execute(*args, **kwargs)

        # TODO: pass tesseract binary path to the pytesseract
        image = Image.open(self.converter.get_page())
        try:
            result = pytesseract.image_to_string(
                image=image, lang=self.language
            )
            # If tesseract gives an error with a language parameter
            # re-run it with no language parameter
        except Exception as exception:
            error_message = 'Exception calling pytesseract with language option: {}; {}'.format(self.language, exception)

            if self.binary:
                if self.language not in self.get_languages():
                    error_message = '{}\nThe requested Tesseract language file for "{}" is not available and needs to be installed.\nIf using Debian or Ubuntu run: apt-get install tesseract-ocr-{}'.format(error_message, self.language, self.language)

            logger.error(error_message)
            raise OCRError(error_message)

        return result



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from rest_framework import serializers

from .models import DocumentMetadata, MetadataType, DocumentTypeMetadataType


class MetadataTypeSerializer(serializers.ModelSerializer):
    class Meta:
        fields = ('id', 'name', 'label', 'default', 'lookup')
        model = MetadataType


class DocumentMetadataSerializer(serializers.ModelSerializer):
    document = serializers.PrimaryKeyRelatedField(read_only=True)

    class Meta:
        fields = ('document', 'id', 'metadata_type', 'value',)
        model = DocumentMetadata
        read_only_fields = ('metadata_type',)


class DocumentTypeMetadataTypeSerializer(serializers.ModelSerializer):
    class Meta:
        fields = ('metadata_type', )
        model = DocumentTypeMetadataType


class DocumentNewMetadataSerializer(serializers.Serializer):
    metadata_type_pk = serializers.IntegerField(
        help_text=_('Primary key of the metadata type to be added.'),
        write_only=True
    )

    metadata_type = MetadataTypeSerializer(read_only=True)

    pk = serializers.IntegerField(
        help_text=_('Primary key of the document metadata type.'),
        read_only=True
    )

    value = serializers.CharField(
        max_length=255,
        help_text=_('Value of the corresponding metadata type instance.')
    )

    def create(self, validated_data):
        metadata_type = MetadataType.objects.get(
            pk=validated_data['metadata_type_pk']
        )
        instance = self.document.metadata.create(
            metadata_type=metadata_type, value=validated_data['value']
        )
        return instance


class DocumentTypeNewMetadataTypeSerializer(serializers.Serializer):
    metadata_type_pk = serializers.IntegerField(
        help_text=_('Primary key of the metadata type to be added.')
    )



from __future__ import unicode_literals

from django.shortcuts import get_object_or_404

from converter.exceptions import UnkownConvertError, UnknownFileFormat
from converter.models import Transformation
from rest_framework import generics
from rest_framework.response import Response

from documents.settings import setting_display_size

from .models import StagingFolderSource
from .serializers import (
    StagingFolderFileSerializer,
    StagingFolderSerializer, StagingSourceFileImageSerializer
)


class APIStagingSourceFileView(generics.GenericAPIView):
    """
    Details of the selected staging file.
    """
    serializer_class = StagingFolderFileSerializer

    def get(self, request, staging_folder_pk, encoded_filename):
        staging_folder = get_object_or_404(
            StagingFolderSource, pk=staging_folder_pk
        )
        return Response(
            StagingFolderFileSerializer(
                staging_folder.get_file(encoded_filename=encoded_filename),
                context={'request': request}
            ).data
        )


class APIStagingSourceListView(generics.ListAPIView):
    """
    Returns a list of all the staging folders and the files they contain.
    """

    serializer_class = StagingFolderSerializer
    queryset = StagingFolderSource.objects.all()


class APIStagingSourceView(generics.RetrieveAPIView):
    """
    Details of the selected staging folders and the files it contains.
    """
    serializer_class = StagingFolderSerializer
    queryset = StagingFolderSource.objects.all()


class APIStagingSourceFileImageView(generics.GenericAPIView):
    """
    Image of the selected staging file.
    size -- 'x' seprated width and height of the desired image representation.
    page -- Page number of the staging file to be imaged.
    zoom -- Zoom level of the image to be generated, numeric value only.
    """

    serializer_class = StagingSourceFileImageSerializer

    def get(self, request, staging_folder_pk, encoded_filename):
        staging_folder = get_object_or_404(
            StagingFolderSource, pk=staging_folder_pk
        )
        staging_file = staging_folder.get_file(
            encoded_filename=encoded_filename
        )

        size = request.GET.get('size', setting_display_size.value)

        try:
            return Response({
                'status': 'success',
                'data': staging_file.get_image(
                    as_base64=True, size=size,
                    transformations=Transformation.objects.get_for_model(
                        staging_folder, as_classes=True
                    )
                )
            })
        except UnknownFileFormat as exception:
            return Response(
                {
                    'status': 'error', 'detail': 'unknown_file_format',
                    'message': unicode(exception)
                }
            )
        except UnkownConvertError as exception:
            return Response(
                {
                    'status': 'error', 'detail': 'converter_error',
                    'message': unicode(exception)
                }
            )



from __future__ import unicode_literals

import logging

from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.db.models import Max
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from .classes import BaseTransformation
from .managers import TransformationManager
from .validators import YAMLValidator

logger = logging.getLogger(__name__)


@python_2_unicode_compatible
class Transformation(models.Model):
    """
    Model that stores the transformation and transformation arguments
    for a given object
    """

    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')

    order = models.PositiveIntegerField(
        blank=True, db_index=True, default=0, help_text=_(
            'Order in which the transformations will be executed. If left '
            'unchanged, an automatic order value will be assigned.'
        ), verbose_name=_('Order')
    )
    name = models.CharField(
        choices=BaseTransformation.get_transformation_choices(),
        max_length=128, verbose_name=_('Name')
    )
    arguments = models.TextField(
        blank=True, help_text=_(
            'Enter the arguments for the transformation as a YAML '
            'dictionary. ie: {"degrees": 180}'
        ), validators=[YAMLValidator()], verbose_name=_('Arguments')
    )

    objects = TransformationManager()

    def __str__(self):
        return self.get_name_display()

    def save(self, *args, **kwargs):
        if not self.order:
            last_order = Transformation.objects.filter(
                content_type=self.content_type, object_id=self.object_id
            ).aggregate(Max('order'))['order__max']
            if last_order is not None:
                self.order = last_order + 1
        super(Transformation, self).save(*args, **kwargs)

    class Meta:
        ordering = ('order',)
        unique_together = ('content_type', 'object_id', 'order')
        verbose_name = _('Transformation')
        verbose_name_plural = _('Transformations')



from __future__ import unicode_literals

from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse
from django.test.client import Client
from django.test import TestCase

from documents.models import DocumentType
from documents.tests.literals import (
    TEST_DOCUMENT_TYPE, TEST_SMALL_DOCUMENT_PATH
)
from user_management.tests import (
    TEST_ADMIN_PASSWORD, TEST_ADMIN_USERNAME, TEST_ADMIN_EMAIL
)

from ..models import Workflow, WorkflowState, WorkflowTransition

from .literals import (
    TEST_WORKFLOW_LABEL, TEST_WORKFLOW_INITIAL_STATE_LABEL,
    TEST_WORKFLOW_INITIAL_STATE_COMPLETION, TEST_WORKFLOW_STATE_LABEL,
    TEST_WORKFLOW_STATE_COMPLETION, TEST_WORKFLOW_TRANSITION_LABEL
)


class DocumentStateViewTestCase(TestCase):
    def setUp(self):
        self.admin_user = get_user_model().objects.create_superuser(
            username=TEST_ADMIN_USERNAME, email=TEST_ADMIN_EMAIL,
            password=TEST_ADMIN_PASSWORD
        )
        self.client = Client()
        # Login the admin user
        logged_in = self.client.login(
            username=TEST_ADMIN_USERNAME, password=TEST_ADMIN_PASSWORD
        )
        self.assertTrue(logged_in)
        self.assertTrue(self.admin_user.is_authenticated())

        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE
        )

        with open(TEST_SMALL_DOCUMENT_PATH) as file_object:
            self.document = self.document_type.new_document(
                file_object=file_object
            )

    def tearDown(self):
        self.document_type.delete()

    def test_creating_workflow(self):
        response = self.client.post(
            reverse(
                'document_states:setup_workflow_create'
            ), data={
                'label': TEST_WORKFLOW_LABEL,
            }, follow=True
        )

        self.assertEquals(response.status_code, 200)

        self.assertEquals(Workflow.objects.count(), 1)
        self.assertEquals(Workflow.objects.all()[0].label, TEST_WORKFLOW_LABEL)

    def test_delete_workflow(self):
        workflow = Workflow.objects.create(label=TEST_WORKFLOW_LABEL)

        self.assertEquals(Workflow.objects.count(), 1)
        self.assertEquals(Workflow.objects.all()[0].label, TEST_WORKFLOW_LABEL)

        response = self.client.post(
            reverse(
                'document_states:setup_workflow_delete', args=(workflow.pk,)
            ), follow=True
        )

        self.assertEquals(response.status_code, 200)

        self.assertEquals(Workflow.objects.count(), 0)

    def test_create_workflow_state(self):
        workflow = Workflow.objects.create(label=TEST_WORKFLOW_LABEL)

        response = self.client.post(
            reverse(
                'document_states:setup_workflow_state_create',
                args=(workflow.pk,)
            ), data={
                'label': TEST_WORKFLOW_STATE_LABEL,
                'completion': TEST_WORKFLOW_STATE_COMPLETION,
            }, follow=True
        )

        self.assertEquals(response.status_code, 200)

        self.assertEquals(WorkflowState.objects.count(), 1)
        self.assertEquals(
            WorkflowState.objects.all()[0].label, TEST_WORKFLOW_STATE_LABEL
        )
        self.assertEquals(
            WorkflowState.objects.all()[0].completion,
            TEST_WORKFLOW_STATE_COMPLETION
        )

    def test_delete_workflow_state(self):
        workflow = Workflow.objects.create(label=TEST_WORKFLOW_LABEL)
        workflow_state = WorkflowState.objects.create(
            workflow=workflow, label=TEST_WORKFLOW_STATE_LABEL,
            completion=TEST_WORKFLOW_STATE_COMPLETION
        )

        response = self.client.post(
            reverse(
                'document_states:setup_workflow_state_delete',
                args=(workflow_state.pk,)
            ), follow=True
        )

        self.assertEquals(response.status_code, 200)

        self.assertEquals(WorkflowState.objects.count(), 0)
        self.assertEquals(Workflow.objects.count(), 1)

    def test_create_workflow_transition(self):
        workflow = Workflow.objects.create(label=TEST_WORKFLOW_LABEL)
        workflow_initial_state = WorkflowState.objects.create(
            workflow=workflow, label=TEST_WORKFLOW_INITIAL_STATE_LABEL,
            completion=TEST_WORKFLOW_INITIAL_STATE_COMPLETION, initial=True
        )
        workflow_state = WorkflowState.objects.create(
            workflow=workflow, label=TEST_WORKFLOW_STATE_LABEL,
            completion=TEST_WORKFLOW_STATE_COMPLETION
        )

        response = self.client.post(
            reverse(
                'document_states:setup_workflow_transition_create',
                args=(workflow.pk,)
            ), data={
                'label': TEST_WORKFLOW_TRANSITION_LABEL,
                'origin_state': workflow_initial_state.pk,
                'destination_state': workflow_state.pk,
            }, follow=True
        )

        self.assertEquals(response.status_code, 200)

        self.assertEquals(WorkflowTransition.objects.count(), 1)
        self.assertEquals(
            WorkflowTransition.objects.all()[0].label,
            TEST_WORKFLOW_TRANSITION_LABEL
        )
        self.assertEquals(
            WorkflowTransition.objects.all()[0].origin_state,
            workflow_initial_state
        )
        self.assertEquals(
            WorkflowTransition.objects.all()[0].destination_state,
            workflow_state
        )

    def test_delete_workflow_transition(self):
        workflow = Workflow.objects.create(label=TEST_WORKFLOW_LABEL)
        workflow_initial_state = WorkflowState.objects.create(
            workflow=workflow, label=TEST_WORKFLOW_INITIAL_STATE_LABEL,
            completion=TEST_WORKFLOW_INITIAL_STATE_COMPLETION, initial=True
        )
        workflow_state = WorkflowState.objects.create(
            workflow=workflow, label=TEST_WORKFLOW_STATE_LABEL,
            completion=TEST_WORKFLOW_STATE_COMPLETION
        )
        workflow_transition = WorkflowTransition.objects.create(
            workflow=workflow, label=TEST_WORKFLOW_TRANSITION_LABEL,
            origin_state=workflow_initial_state,
            destination_state=workflow_state
        )

        self.assertEquals(WorkflowTransition.objects.count(), 1)

        response = self.client.post(
            reverse(
                'document_states:setup_workflow_transition_delete',
                args=(workflow_transition.pk,)
            ), follow=True
        )

        self.assertEquals(response.status_code, 200)

        self.assertEquals(WorkflowState.objects.count(), 2)
        self.assertEquals(Workflow.objects.count(), 1)
        self.assertEquals(WorkflowTransition.objects.count(), 0)



from __future__ import unicode_literals

from django.apps import apps
from django.conf import settings
from django.contrib import messages
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.utils.translation import ungettext, ugettext_lazy as _

from permissions import Permission

__all__ = (
    'DeleteExtraDataMixin', 'ExtraContextMixin',
    'ObjectListPermissionFilterMixin', 'ObjectNameMixin',
    'ObjectPermissionCheckMixin', 'RedirectionMixin',
    'ViewPermissionCheckMixin'
)


class DeleteExtraDataMixin(object):
    def delete(self, request, *args, **kwargs):
        self.object = self.get_object()
        success_url = self.get_success_url()
        if hasattr(self, 'get_delete_extra_data'):
            self.object.delete(**self.get_delete_extra_data())
        else:
            self.object.delete()

        return HttpResponseRedirect(success_url)


class ExtraContextMixin(object):
    extra_context = {}

    def get_extra_context(self):
        return self.extra_context

    def get_context_data(self, **kwargs):
        context = super(ExtraContextMixin, self).get_context_data(**kwargs)
        context.update(self.get_extra_context())
        return context


class MultipleInstanceActionMixin(object):
    model = None
    success_message = 'Operation performed on %(count)d object'
    success_message_plural = 'Operation performed on %(count)d objects'

    def get_pk_list(self):
        return self.request.GET.get(
            'id_list', self.request.POST.get('id_list', '')
        ).split(',')

    def get_queryset(self):
        return self.model.objects.filter(pk__in=self.get_pk_list())

    def get_success_message(self, count):
        return ungettext(
            self.success_message,
            self.success_message_plural,
            count
        ) % {
            'count': count,
        }

    def post(self, request, *args, **kwargs):
        count = 0
        for instance in self.get_queryset():
            try:
                self.object_action(instance=instance)
            except PermissionDenied:
                pass
            else:
                count += 1

        messages.success(
            self.request,
            self.get_success_message(count=count)
        )

        return HttpResponseRedirect(self.get_success_url())


class ObjectListPermissionFilterMixin(object):
    object_permission = None

    def get_queryset(self):
        AccessControlList = apps.get_model(
            app_label='acls', model_name='AccessControlList'
        )

        queryset = super(ObjectListPermissionFilterMixin, self).get_queryset()

        if self.object_permission:
            try:
                # Check to see if the user has the permissions globally
                Permission.check_permissions(
                    self.request.user, (self.object_permission,)
                )
            except PermissionDenied:
                # No global permission, filter ther queryset per object +
                # permission
                return AccessControlList.objects.filter_by_access(
                    self.object_permission, self.request.user, queryset
                )
            else:
                # Has the permission globally, return all results
                return queryset
        else:
            return queryset


class ObjectNameMixin(object):
    def get_object_name(self, context=None):
        if not context:
            context = self.get_context_data()

        object_name = context.get('object_name')

        if not object_name:
            try:
                object_name = self.object._meta.verbose_name
            except AttributeError:
                object_name = _('Object')

        return object_name


class ObjectPermissionCheckMixin(object):
    object_permission = None

    def get_permission_object(self):
        return self.get_object()

    def dispatch(self, request, *args, **kwargs):
        AccessControlList = apps.get_model(
            app_label='acls', model_name='AccessControlList'
        )

        if self.object_permission:
            try:
                Permission.check_permissions(
                    request.user, (self.object_permission,)
                )
            except PermissionDenied:
                AccessControlList.objects.check_access(
                    self.object_permission, request.user,
                    self.get_permission_object(),
                    related=getattr(self, 'object_permission_related', None)
                )

        return super(
            ObjectPermissionCheckMixin, self
        ).dispatch(request, *args, **kwargs)


class RedirectionMixin(object):
    post_action_redirect = None
    action_cancel_redirect = None

    def get_post_action_redirect(self):
        return self.post_action_redirect

    def get_action_cancel_redirect(self):
        return self.action_cancel_redirect

    def dispatch(self, request, *args, **kwargs):
        post_action_redirect = self.get_post_action_redirect()
        action_cancel_redirect = self.get_action_cancel_redirect()

        self.next_url = self.request.POST.get(
            'next', self.request.GET.get(
                'next', post_action_redirect if post_action_redirect else self.request.META.get(
                    'HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL)
                )
            )
        )
        self.previous_url = self.request.POST.get(
            'previous', self.request.GET.get(
                'previous', action_cancel_redirect if action_cancel_redirect else self.request.META.get(
                    'HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL)
                )
            )
        )

        return super(
            RedirectionMixin, self
        ).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(RedirectionMixin, self).get_context_data(**kwargs)
        context.update(
            {
                'next': self.next_url,
                'previous': self.previous_url
            }
        )

        return context

    def get_success_url(self):
        return self.next_url or self.previous_url


class ViewPermissionCheckMixin(object):
    view_permission = None

    def dispatch(self, request, *args, **kwargs):
        if self.view_permission:
            Permission.check_permissions(
                requester=self.request.user,
                permissions=(self.view_permission,)
            )

        return super(
            ViewPermissionCheckMixin, self
        ).dispatch(request, *args, **kwargs)
